import eol from "eol";
import fs from "fs";
import path from "path";
import resolve from "resolve";
import slash from "slash";
import { fileURLToPath, pathToFileURL } from "url";
import { removeTrailingForwardSlash } from "./path.js";
function isObject(value) {
  return typeof value === "object" && value != null;
}
function arraify(target) {
  return Array.isArray(target) ? target : [target];
}
function padMultilineString(source, n = 2) {
  const lines = source.split(/\r?\n/);
  return lines.map((l) => ` `.repeat(n) + l).join(`
`);
}
const STATUS_CODE_REGEXP = /^\/?[0-9]{3}$/;
function getOutputFilename(astroConfig, name) {
  if (name === "/" || name === "") {
    return path.posix.join(name, "index.html");
  }
  if (astroConfig.build.format === "directory" && !STATUS_CODE_REGEXP.test(name)) {
    return path.posix.join(name, "index.html");
  }
  return `${removeTrailingForwardSlash(name || "index")}.html`;
}
function parseNpmName(spec) {
  if (!spec || spec[0] === "." || spec[0] === "/")
    return void 0;
  let scope;
  let name = "";
  let parts = spec.split("/");
  if (parts[0][0] === "@") {
    scope = parts[0];
    name = parts.shift() + "/";
  }
  name += parts.shift();
  let subpath = parts.length ? `./${parts.join("/")}` : void 0;
  return {
    scope,
    name,
    subpath
  };
}
function createSafeError(err) {
  return err instanceof Error || err && err.name && err.message ? err : new Error(JSON.stringify(err));
}
function codeFrame(src, loc) {
  if (!loc)
    return "";
  const lines = eol.lf(src).split("\n");
  const visibleLines = [];
  for (let n = -2; n <= 2; n++) {
    if (lines[loc.line + n])
      visibleLines.push(loc.line + n);
  }
  let gutterWidth = 0;
  for (const lineNo of visibleLines) {
    let w = `> ${lineNo}`;
    if (w.length > gutterWidth)
      gutterWidth = w.length;
  }
  let output = "";
  for (const lineNo of visibleLines) {
    const isFocusedLine = lineNo === loc.line - 1;
    output += isFocusedLine ? "> " : "  ";
    output += `${lineNo + 1} | ${lines[lineNo]}
`;
    if (isFocusedLine)
      output += `${[...new Array(gutterWidth)].join(" ")}  | ${[...new Array(loc.column)].join(" ")}^
`;
  }
  return output;
}
function resolveDependency(dep, astroConfig) {
  const resolved = resolve.sync(dep, {
    basedir: fileURLToPath(astroConfig.root)
  });
  return pathToFileURL(resolved).toString();
}
function viteID(filePath) {
  return slash(fileURLToPath(filePath));
}
const VALID_ID_PREFIX = `/@id/`;
function unwrapId(id) {
  return id.startsWith(VALID_ID_PREFIX) ? id.slice(VALID_ID_PREFIX.length) : id;
}
function removeDir(_dir) {
  const dir = fileURLToPath(_dir);
  fs.rmSync(dir, { recursive: true, force: true, maxRetries: 3 });
}
function emptyDir(_dir, skip) {
  const dir = fileURLToPath(_dir);
  if (!fs.existsSync(dir))
    return void 0;
  for (const file of fs.readdirSync(dir)) {
    if (skip == null ? void 0 : skip.has(file)) {
      continue;
    }
    fs.rmSync(path.resolve(dir, file), { recursive: true, force: true, maxRetries: 3 });
  }
}
function resolvePages(config) {
  return new URL("./pages", config.srcDir);
}
function isBuildingToSSR(config) {
  const adapter = config._ctx.adapter;
  if (!adapter)
    return false;
  if (typeof adapter.serverEntrypoint === "string") {
    if (!adapter.name.startsWith("@astrojs/") && !config.experimental.ssr) {
      throw new Error([
        `Server-side rendering (SSR) is still experimental.`,
        ``,
        `Only official "@astrojs/*" adapters are currently supported.`,
        `To enable SSR for 3rd-party adapters, use the "--experimental-ssr" flag.`,
        `Breaking changes may occur in this API before Astro v1.0 is released.`,
        ``
      ].join("\n"));
    } else {
      return true;
    }
  } else {
    return false;
  }
}
function emoji(char, fallback) {
  return process.platform !== "win32" ? char : fallback;
}
function getLocalAddress(serverAddress, host) {
  if (typeof host === "boolean" || host === "localhost") {
    return "localhost";
  } else {
    return serverAddress;
  }
}
export {
  VALID_ID_PREFIX,
  arraify,
  codeFrame,
  createSafeError,
  emoji,
  emptyDir,
  getLocalAddress,
  getOutputFilename,
  isBuildingToSSR,
  isObject,
  padMultilineString,
  parseNpmName,
  removeDir,
  resolveDependency,
  resolvePages,
  unwrapId,
  viteID
};
